"use strict";

function ScatterPlot(scene)
{
	this.showaxislabels = true;
	this.showpolarization = true;
	this.showrllabels = false;
	this.showtransmitted = false;
	this.showsphere = false;
	this.arrowlength = 1.;
	this.fixMiller = false;
	// light orientation
	this.theta = 120.;
	this.phi = 45.;
	this.omega = 0.;
	
	// sample orientation
	this.chi = 90.;
	this.eta = 0.;
	
	// for diffraction
	this.psi = 0.;
	
	this.arrowthickness = 0.1;
	
	this.radius = 3;
	this.scene = scene;
	this.objects = [];
	//this.SetEnergy(5000.);
	//this.SetEnergy(1650.);
	//this.SetEnergy(8350.);
	//this.SetEnergy(853.);
	this.SetEnergy(5000);
	this.G = [];
	this.Gk = [];
	//this.diffh = -1.;
	//this.diffk = -1.;
	//this.diffl = 1.;
	this.diffh = 0.;
	this.diffk = 0.;
	this.diffl = 1.;
	
/*
	this.a1 = {x:10.62360, y:0., z:0.};
	this.a2 = {x:0., y:10.62360, z:0.};
	this.a3 = {x:0., y:0., z:3.857*4};
*/

	this.a1 = {x:4., y:0., z:0.};
	this.a2 = {x:0., y:4., z:0.};
	this.a3 = {x:0., y:0., z:4.};

/*
	this.a1 = {x:4.*3.84, y:0., z:0.};
	this.a2 = {x:0., y:4.*3.84, z:0.};
	this.a3 = {x:0., y:0., z:4.*3.84};
*/
/*
	this.a1 = {x:5.47, y:0., z:0.};
	this.a2 = {x:0., y:5.47, z:0.};
	this.a3 = {x:0., y:0., z:3.78*2.};
*/
/*
	this.a1 = {x:8., y:0., z:0.};
	this.a2 = {x:0., y:8., z:0.};
	this.a3 = {x:0., y:0., z:8.};
*/
	this.materialred = new THREE.MeshLambertMaterial(
	{
		color: 'red',
		ambient: 'red',
		specular: 'white',
		//shading: THREE.FlatShading,
	});
	this.materialorange = new THREE.MeshLambertMaterial(
	{
		color: 'orange',
		ambient: 'orange',
		specular: 'white',
		//shading: THREE.FlatShading,
	});

	this.materialblack = new THREE.MeshLambertMaterial(
	{
		color: 'black',
		ambient: 'black',
	});

	this.materialgrey = new THREE.MeshLambertMaterial(
	{
		color: '#808080',
		ambient: '#808080',
	});
}

ScatterPlot.prototype.SetLattice = function(a1,a2,a3,alpha,beta,gamma) {
	alpha *=  Math.PI / 180.;
	beta  *=  Math.PI / 180.;
	gamma *=  Math.PI / 180.;
	// http://en.wikipedia.org/wiki/Fractional_coordinates
	this.a1.x = a1;
	this.a1.y = 0.;
	this.a1.z = 0.;
	this.a2.x = a2*Math.cos(gamma);
	this.a2.y = a2*Math.sin(gamma);
	this.a2.z = 0.;
	var v = Math.sqrt(
		1. - Math.cos(alpha)*Math.cos(alpha) - Math.cos(beta)*Math.cos(beta) - Math.cos(gamma)*Math.cos(gamma)
		   + 2.*Math.cos(alpha)*Math.cos(beta)*Math.cos(gamma)
	);
	this.a3.x = a3 * Math.cos(beta);
	this.a3.y = a3 * (Math.cos(alpha) - Math.cos(beta)*Math.cos(gamma)) / Math.sin(gamma);
	this.a3.z = a3 * v/Math.sin(gamma);
	console.log(this.a1);
	console.log(this.a2);
	console.log(this.a3);
	this.Update();
}

ScatterPlot.prototype.PlotDiffCircle = function() {
	var group = new THREE.Object3D();

	this.CreateReciprocalLattice();
	for(this.phi=0; this.phi<360; this.phi+=0.1)
	{
		var ret = this.SetThetaHKL(this.diffh, this.diffk, this.diffl);
		if (!ret) continue;
		this.CalcLightWavevector();
		
		for(var i=0; i<this.G.length; i++)
		{
			var koutx = this.k0*this.kx + this.G[i].x;
			var kouty = this.k0*this.ky + this.G[i].y;
			var det = this.k0*this.k0 - koutx*koutx - kouty*kouty;
			if (det < 0) continue;
			var koutz = Math.sqrt(det);
			var dir = new THREE.Vector3(koutx, kouty, koutz);
			dir.normalize();
			var origin = new THREE.Vector3( 0., 0., this.arrowthickness/2 );
			length = 1;
			hex = 0xff8000;
			dir.multiplyScalar(this.arrowlength);
			var geometry = new THREE.SphereGeometry( 0.01, 5, 5 ); 
			var sphere = new THREE.Mesh( geometry, materialgrey ); 
			sphere.position.set(dir.x, dir.y, dir.z);
			group.add(sphere);
		}
	}
	scene.add(group);
}

ScatterPlot.prototype.SetEnergy = function(eng) 
{
	this.energy = eng;
	this.lambda = 12398.326 / this.energy; // in A
	this.k0 = 2. * Math.PI / this.lambda;
}

ScatterPlot.prototype.Remove = function(object) 
{
	var obj, i;
	for ( i = this.scene.children.length - 1; i >= 0 ; i -- ) 
	{
		obj = this.scene.children[ i ];
		if ( obj==object) 
		{
			this.scene.remove(obj);
		}
	}
}

ScatterPlot.prototype.DeleteArrows = function()
{
	
	for (var i=0; i<this.objects.length; i++)
		this.Remove(this.objects[i]);
	this.objects = [];
}

ScatterPlot.prototype.CreateReciprocalLattice = function()
{
	this.G = [];
	this.Ni = [];
	this.Nj = [];

	var A0 = Math.abs(this.a1.x*this.a2.y - this.a1.y*this.a2.x);
	var ra0 = 2. * Math.PI / A0;
	
	this.b1 = {x: this.a2.y*ra0, y:-this.a2.x*ra0, z:0.};
	this.b2 = {x:-this.a1.y*ra0, y: this.a1.x*ra0, z:0.};
	this.b3 = {x:0., y: 0., z: 2. * Math.PI/this.a3.z};
	
	for(var j=-12; j<=12; j++)
	for(var i=-12; i<=12; i++) {
		if (this.fixMiller)
		{
			if (i != this.diffh) continue;
			if (j != this.diffk) continue;
		}
		var v = {x:i*this.b1.x + j*this.b2.x, y:i*this.b1.y + j*this.b2.y};
		var length = (v.x*v.x + v.y*v.y);
		if (length > this.radius*this.radius) continue;
		this.G.push(v);
		this.Ni.push(i);
		this.Nj.push(j);
	}
}

ScatterPlot.prototype.SetThetaHKL = function(h, k, l)
{
	var g = {
		x: h*this.b1.x + k*this.b2.x + l*this.b3.x, 
		y: h*this.b1.y + k*this.b2.y + l*this.b3.y, 
		z: h*this.b1.z + k*this.b2.z + l*this.b3.z
		};

var phi = this.phi/180.*Math.PI;

var theta1 = 0.;
{
var cg1 = g.x/this.k0;
var cg3 = g.y/this.k0;
var cg5 = g.z/this.k0;
var t2 = Math.sin(phi);
var t3 = t2 * cg3;
var t4 = cg5*cg5;
var t6 = cg1*cg1;
var t8 = Math.cos(phi);
var t11 = cg3 * cg3;
var t20 = t4 * t4;
var t21 = t8 * t8;
var t22 = t11 * t21;
var t33 = cg1 * t8;
var t37 = t6 * t6;
var t42 = t11 * t11;
var t43 = -2. * t11 * t4 - t20 - 4. * t22 + 4. * t6 * t21 + 4. * t4 + 4. * t11 + 8. * t33 * t3 - t37 - 2. * t4 * t6 - 2. * t11 * t6 - t42;
var t44 = Math.sqrt(t43);
var t45 = Math.abs(cg5);
var t53 = 1.;
var t59 = (cg5>0 ? 1 : (cg5<0 ? -1 : 0));
var t60 = t44 * t59;
var t67 = Math.atan2(	-t53 * (t4 * t3 + t6 * t3 + cg1 * t6 * t8 + t2 * cg3 * t11 + cg1 * t8 * t11 + cg1 * t8 * t4 + t45 * t44) / 2., 
						 t53 * (-cg5 * t4 - t6 * cg5 - t11 * cg5 + t60 * t3 + t60 * t33) / 2.);
theta1 = t67;
}

var theta2 = 0.;
{
var cg1 = g.x/this.k0;
var cg3 = g.y/this.k0;
var cg5 = g.z/this.k0;
var t2 = Math.sin(phi);
var t3 = t2 * cg3;
var t4 = cg5*cg5;
var t6 = cg1*cg1;
var t8 = Math.cos(phi);
var t11 = cg3 * cg3;
var t20 = t4 * t4;
var t21 = t8 * t8;
var t22 = t11 * t21;
var t33 = cg1 * t8;
var t37 = t6 * t6;
var t42 = t11 * t11;
var t43 = -2. * t11 * t4 - t20 - 4. * t22 + 4. * t6 * t21 + 4. * t4 + 4. * t11 + 8. * t33 * t3 - t37 - 2. * t4 * t6 - 2. * t11 * t6 - t42;
var t44 = Math.sqrt(t43);
var t45 = Math.abs(cg5);
var t53 = 1.;
var t59 = (cg5>0. ? 1. : (cg5<0 ? -1. : 0.));
var t60 = t44 * t59;
var t67 = Math.atan2(	-t53 * (t4 * t3 + t6 * t3 + cg1 * t6 * t8 + t2 * cg3 * t11 + cg1 * t8 * t11 + cg1 * t8 * t4 - t45 * t44) / 2., 
						-t53 * (cg5 * t4 + t6 * cg5 + t11 * cg5 + t60 * t3 + t60 * t33) / 2.);
theta2 = t67;
}
var thetatemp = this.theta;

var valid1 = true;
if (isNaN(theta1)) valid1 = false; 
else 
{
	if (theta1 < Math.PI/2.) theta1 += 2.*Math.PI;
	if (theta1 > Math.PI*3./2.) valid1 = false;
}

var valid2 = true;
if (isNaN(theta2)) valid2 = false; 
else
{
	if (theta2 < Math.PI/2.) theta2 += 2.*Math.PI;
	if (theta2 > Math.PI*3./2.) valid2 = false;
}

if ((!valid1) && (!valid2)) return false;

if (valid1)
	this.theta = theta1*180. / Math.PI;

if (valid2)
	this.theta = theta2*180. / Math.PI;

	//console.log("theta="  + (theta1*180./Math.PI) + " " + (theta2*180./Math.PI));

	this.CalcLightWavevector();
	var kout = this.CalcKout(h, k, l);

	var qx = (kout.x - this.k0*this.kx);
	var qy = (kout.y - this.k0*this.ky);
	var qz = (-kout.z - this.k0*this.kz); // in reflection
	if (l < 0) // in transmission
	{
		qz = (kout.z - this.k0*this.kz);
	}
	var check = (qx-g.x)*(qx-g.x) + (qy-g.y)*(qy-g.y) + (qz-g.z)*(qz-g.z);

	//console.log(check);
	
	if (check > 1e-5) {
		this.theta = thetatemp;
		return false;
	}
	$("#thetarange").val(this.theta);
	$("#thetalabel").text(this.theta);
	return true;
}

ScatterPlot.prototype.CalcLightWavevector = function()
{
	this.kx = Math.sin(this.theta * Math.PI / 180.) * Math.cos(this.phi * Math.PI / 180.);
	this.ky = Math.sin(this.theta * Math.PI / 180.) * Math.sin(this.phi * Math.PI / 180.);
	this.kz = Math.cos(this.theta * Math.PI / 180.);
}

ScatterPlot.prototype.CalcKout = function(h, k, l)
{
	var g = {
		x: h*this.b1.x + k*this.b2.x + l*this.b3.x,
		y: h*this.b1.y + k*this.b2.y + l*this.b3.y,
		z: h*this.b1.z + k*this.b2.z + l*this.b3.z
		};
	var koutx = this.k0*this.kx + g.x;
	var kouty = this.k0*this.ky + g.y;
	var det = this.k0*this.k0 - koutx*koutx - kouty*kouty;
	if (det < 0) return {x:0, y:0, z:0};
	var koutz = Math.sqrt(det);
	if (this.kz < 0) koutz = -koutz;
	return {x:koutx, y:kouty, z:koutz};
}

ScatterPlot.prototype.CreateAxisLabels = function()
{
	var spritex = makeTextSprite( "x", { fontsize: 80 } );
	spritex.position.set(0.62, 0, 0);
	scene.add( spritex );

	var spritey = makeTextSprite( "y", { fontsize: 80 } );
	spritey.position.set(0.0, 0.62, 0);
	scene.add( spritey );

	this.objects.push(spritex);
	this.objects.push(spritey);
}

function NormVector(v)
{
	var r = v.x*v.x + v.y*v.y + v.z*v.z;
	v.x = v.x / Math.sqrt(r);
	v.y = v.y / Math.sqrt(r);
	v.z = v.z / Math.sqrt(r);

}

ScatterPlot.prototype.CreateScatterSphere = function(h, k, l)
{
	var group = new THREE.Object3D();

	var g = {
		x: h*this.b1.x + k*this.b2.x + l*this.b3.x, 
		y: h*this.b1.y + k*this.b2.y + l*this.b3.y, 
		z: h*this.b1.z + k*this.b2.z + l*this.b3.z
		};
	var v1 = {x: 1./Math.sqrt(2.), y: 1./Math.sqrt(2.), z: 0.};
	//var v1 = {x: 1., y: 0., z: 0.};
	
	var s = (g.x*v1.x + g.y*v1.y + g.z*v1.z) / (g.x*g.x + g.y*g.y + g.z*g.z);
	v1.x -= s*g.x;
	v1.y -= s*g.y;
	v1.z -= s*g.z;
	//console.log(g);
	//console.log(v1);
	//console.log(g.x*v1.x + g.y*v1.y + g.z*v1.z);
	var v2 = {x: 0., y: 0., z: 0.};
	v2.x = g.y*v1.z - g.z*v1.y;
	v2.y = g.z*v1.x - g.x*v1.z;
	v2.z = g.x*v1.y - g.y*v1.x;
	var v3 = {x:g.x, y:g.y, z:g.z};
	NormVector(v1);
	NormVector(v2);
	NormVector(v3);
	
	// calc radius
	var sinpsi = g.z/this.k0/2./v3.z; // psi in special coordinate system
	if ((sinpsi < -1) || (sinpsi > 1)) return;
	
	var a2 = Math.sqrt(1.-sinpsi*sinpsi); // cos
	var b2 = sinpsi;                   // and sin

	// --------------------
	var origin = new THREE.Vector3(b2*v3.x*0.5, b2*v3.y*0.5, b2*v3.z*0.5);
	
	var dir = new THREE.Vector3(v1.x, v1.y, v1.z);
	var cyl = CreateCylinder(dir, origin, a2*0.5, 2.5, "#808080");
	group.add(cyl);
	
	var dir = new THREE.Vector3(-v1.x, -v1.y, -v1.z);
	var cyl = CreateCylinder(dir, origin, a2*0.5, 2.5, "#808080")
	group.add(cyl);
	
	var dir = new THREE.Vector3(v2.x, v2.y, v2.z);
	var cyl = CreateCylinder(dir, origin, a2*0.5, 2.5, "#808080")
	group.add(cyl);
	
	var dir = new THREE.Vector3(-v2.x, -v2.y, -v2.z);	
	var cyl = CreateCylinder(dir, origin, a2*0.5, 2.5, "#808080");
	group.add(cyl);
	
	/*
	var sprite = makeTextSprite( "0", { fontsize: 80 } );
	sprite.position.set((b2*v3.x + a2*v1.x*1.2)*0.5, (b2*v3.y + a2*v1.y*1.2)*0.5, (b2*v3.z + a2*v1.z*1.2)*0.5);
	scene.add( sprite );

	var sprite = makeTextSprite( "180", { fontsize: 80 } );
	sprite.position.set((b2*v3.x - a2*v1.x*1.2)*0.5, (b2*v3.y - a2*v1.y*1.2)*0.5, (b2*v3.z - a2*v1.z*1.2)*0.5);
	scene.add( sprite );
*/
	var lastvalid = false;
	
	for(var phi=0; phi<=360; phi+=5) 
	{
		var a = Math.cos(phi/180.*3.141592);
		var b = Math.sin(phi/180.*3.141592);
		//var r = 1.* g.x/(this.k0);
		// we calculate kin-kout in special coordinate system
		var vperp = v3;
		var vpar = {
			x:a*v1.x + b*v2.x, 
			y:a*v1.y + b*v2.y, 
			z:a*v1.z + b*v2.z
			};
		
		// definitions
		// kin = cos(psi)*vpar + sin(psi)*vperp
		// kout = cos(psi)*vpar - sin(psi)*vperp
		
		if (( a2*vpar.z + b2*vperp.z) < 0) {lastvalid = false; continue;}
		if ((-a2*vpar.z + b2*vperp.z) < 0) {lastvalid = false; continue;}
		/*
		var geometry = new THREE.SphereGeometry( 0.01, 5, 5 );
		var sphere = new THREE.Mesh( geometry, this.materialgrey );

		sphere.position.set(
			(a2*vpar.x + b2*vperp.x)*0.5, 
			(a2*vpar.y + b2*vperp.y)*0.5, 
			(a2*vpar.z + b2*vperp.z)*0.5);
		scene.add(sphere);
		this.objects.push(sphere);
		*/
		var p = new THREE.Vector3(
			(a2*vpar.x + b2*vperp.x)*0.5, 
			(a2*vpar.y + b2*vperp.y)*0.5, 
			(a2*vpar.z + b2*vperp.z)*0.5);
			
		if (lastvalid) {
			var cyl = CreateCylinder2(plast, p, 0.003, "#808080");
			group.add(cyl);
		}
		var plast = p;
		lastvalid = true;
	}
	this.objects.push(group);
	scene.add(group);
}

ScatterPlot.prototype.SetThetaPhiHKL = function(h, k, l)
{
	var g = {
		x: h*this.b1.x + k*this.b2.x + l*this.b3.x, 
		y: h*this.b1.y + k*this.b2.y + l*this.b3.y, 
		z: h*this.b1.z + k*this.b2.z + l*this.b3.z
		};
	var v1 = {x: 1., y: 0., z: 0.};
	
	var s = (g.x*v1.x + g.y*v1.y + g.z*v1.z) / (g.x*g.x + g.y*g.y + g.z*g.z);
	v1.x -= s*g.x;
	v1.y -= s*g.y;
	v1.z -= s*g.z;
	//console.log(g);
	//console.log(v1);
	//console.log(g.x*v1.x + g.y*v1.y + g.z*v1.z);
	var v2 = {x: 0., y: 0., z: 0.};
	v2.x = g.y*v1.z - g.z*v1.y;
	v2.y = g.z*v1.x - g.x*v1.z;
	v2.z = g.x*v1.y - g.y*v1.x;
	var v3 = {x:g.x, y:g.y, z:g.z};
	NormVector(v1);
	NormVector(v2);
	NormVector(v3);
	
	var a = Math.cos(this.psi/180.*3.141592);
	var b = Math.sin(this.psi/180.*3.141592);
	var rhs = g.x / this.k0;
	
	// vperp and vpar have norm = 1
	var vperp = v3;
	var vpar = {
		x:a*v1.x + b*v2.x,
		y:a*v1.y + b*v2.y,
		z:a*v1.z + b*v2.z
		};
		
		// definitions
		// kin = cos(psi)*vpar + sin(psi)*vperp
		// kout = cos(psi)*vpar - sin(psi)*vperp
		var sinpsi = g.z/this.k0/2./v3.z; // psi in special coordinate system
		if ((sinpsi < -1) || (sinpsi > 1)) return 0;
		
		var a = Math.sqrt(1.-sinpsi*sinpsi); // cos
		var b = sinpsi;                   // and sin
		
		if ((a*vpar.z + b*vperp.z) < 0) return 0; // transmission
		if ((-a*vpar.z + b*vperp.z) < 0) return 0; // transmission
		var p = {
			x:(a*vpar.x + b*vperp.x), 
			y:(a*vpar.y + b*vperp.y), 
			z:(a*vpar.z + b*vperp.z)
			};
		
		this.theta = Math.acos(-p.z)*180./3.141592;
		this.phi   = Math.atan2(-p.y, -p.x)*180./3.141592;
		return 1;
}

function CreateArrow(dir, origin, length, thickness, hex)
{
	var group = new THREE.Object3D();
	var arrow = new THREE.ArrowHelper( dir, origin, length, hex);
	group.add(arrow);
	
	dir.multiplyScalar(length);
	//var rotation = new THREE.Euler().setFromQuaternion(arrow.quaternion);
	
	var mat = new THREE.MeshBasicMaterial( { color: hex } );
	var edgeGeometry = new THREE.CylinderGeometry( length*0.005*thickness, length*0.005*thickness, dir.length()*0.8, 6, 4 );
	var edge = new THREE.Mesh( edgeGeometry, mat);
	group.add(edge);
	//edge.rotation = rotation.clone();
	//edge.quaternion.setRotationFromEuler(rotation.clone());
	var q = arrow.quaternion.clone();
	edge.useQuaterion = true;
	//edge.position = new THREE.Vector3().addVectors( origin, dir.multiplyScalar(0.5) );
	edge.quaternion.set(q.x, q.y, q.z, q.w);
	edge.position.set(origin.x+dir.x*0.4, origin.y+dir.y*0.4, origin.z+dir.z*0.4);

	var material = new THREE.MeshLambertMaterial(
	{
		color: hex,
		ambient: hex,
		//shading: THREE.FlatShading,
	});
	edge.material = material;
	arrow.cone.material = material;
	return group;
}


function CreateCylinder(dir, origin, length, thickness, hex)
{
	var group = new THREE.Object3D();
	var arrow = new THREE.ArrowHelper( dir, origin, length, hex);
	
	dir.multiplyScalar(length);
	//var rotation = new THREE.Euler().setFromQuaternion(arrow.quaternion);
	var mat = new THREE.MeshBasicMaterial( { color: hex } );
	var edgeGeometry = new THREE.CylinderGeometry( length*0.005*thickness, length*0.005*thickness, dir.length()*1.0, 6, 4 );
	var edge = new THREE.Mesh( edgeGeometry, mat);
	group.add(edge);
	//edge.rotation = rotation.clone();
	//edge.quaternion.setRotationFromEuler(rotation.clone());
	var q = arrow.quaternion.clone();
	edge.useQuaterion = true;
	//edge.position = new THREE.Vector3().addVectors( origin, dir.multiplyScalar(0.5) );
	edge.quaternion.set(q.x, q.y, q.z, q.w);
	edge.position.set(origin.x+dir.x*0.5, origin.y+dir.y*0.5, origin.z+dir.z*0.5);

	var material = new THREE.MeshLambertMaterial(
	{
		color: hex,
		ambient: hex,
		//shading: THREE.FlatShading,
	});
	edge.material = material;	
	return group;
}

function CreateCylinder2(pointX, pointY, thickness, hex)
{
	var dir = new THREE.Vector3().subVectors( pointY, pointX );
	var length = dir.length();
	dir.multiplyScalar(1./length);
	var group = new THREE.Object3D();
	var arrow = new THREE.ArrowHelper( dir, pointX, length, hex);

	var mat = new THREE.MeshBasicMaterial( { color: hex } );
	var edgeGeometry = new THREE.CylinderGeometry( thickness, thickness, length*1.0, 6, 4 );
	var edge = new THREE.Mesh( edgeGeometry, mat);
	group.add(edge);
	var q = arrow.quaternion.clone();
	edge.useQuaterion = true;
	edge.quaternion.set(q.x, q.y, q.z, q.w);
	edge.position.set(pointX.x+dir.x*length*0.5, pointX.y+dir.y*length*0.5, pointX.z+dir.z*length*0.5);

	var material = new THREE.MeshLambertMaterial(
	{
		color: hex,
		ambient: hex,
	});
	edge.material = material;	
	return group;
}



/*
var cylinderMesh = function( pointX, pointY )
{
    // edge from X to Y
    var direction = new THREE.Vector3().subVectors( pointY, pointX );
    var arrow = new THREE.ArrowHelper( direction, pointX );

    // cylinder: radiusAtTop, radiusAtBottom, 
    //     height, radiusSegments, heightSegments
    var edgeGeometry = new THREE.CylinderGeometry( 2, 2, direction.length(), 6, 4 );

    var edge = new THREE.Mesh( edgeGeometry, 
        new THREE.MeshBasicMaterial( { color: 0x0000ff } ) );
    edge.rotation = arrow.rotation.clone();
    edge.position = new THREE.Vector3().addVectors( pointX, direction.multiplyScalar(0.5) );
    return edge;
}
*/

ScatterPlot.prototype.CreatePolarization = function()
{
	var origin = new THREE.Vector3( -this.arrowlength*this.kx, -this.arrowlength*this.ky, -this.arrowlength*this.kz/*+this.arrowthickness/2 */);
	var group = new THREE.Object3D();

	var sx = -Math.sin(this.phi * Math.PI / 180.);
	var sy = Math.cos(this.phi * Math.PI / 180.);
	var sz = 0.;

	var dir = new THREE.Vector3( sx, sy, sz );
	dir.normalize();
	var length = 0.2;
	var hex = 0x008000;
	var arrow = CreateArrow( dir, origin, length, 1, hex);
	this.objects.push(arrow);
	group.add( arrow );

	dir = new THREE.Vector3(-sx, -sy, -sz);
	dir.normalize();
	length = 0.2;
	hex = 0x008000;
	arrow = CreateArrow( dir, origin, length, 1, hex);
	group.add(arrow);

	var sx =  Math.cos((this.theta) * Math.PI / 180.) * Math.cos(this.phi * Math.PI / 180.);
	var sy =  Math.cos((this.theta) * Math.PI / 180.) * Math.sin(this.phi * Math.PI / 180.);
	var sz = -Math.sin((this.theta) * Math.PI / 180.);

	var dir = new THREE.Vector3( sx, sy, sz );
	dir.normalize();
	var length = 0.2;
	var hex = 0x008000;
	var arrow = CreateArrow( dir, origin, length, 1, hex);
	group.add( arrow );

	dir = new THREE.Vector3( -sx, -sy, -sz );
	dir.normalize();
	length = 0.2;
	hex = 0x008000;
	arrow = CreateArrow( dir, origin, length, 1, hex);
	group.add( arrow );
	
	this.objects.push(group);
	scene.add(group);
}

ScatterPlot.prototype.CreateSphere = function()
{
	var materialsphere = new THREE.MeshLambertMaterial(
	{
		color: 'white',
		ambient: 'white',
		opacity: 0.2,
		transparent: true
		//shading: THREE.FlatShading,
	});

var geometry = new THREE.SphereGeometry( this.arrowlength, 20, 20 );
var cube = new THREE.Mesh( geometry, materialsphere ); 
cube.position.set(0., 0., 0.);
scene.add(cube); 
this.objects.push(cube);
}

ScatterPlot.prototype.RenderSample = function()
{

var group = new THREE.Object3D();

var material = new THREE.MeshLambertMaterial(
	{
		color: 'blue',
		ambient: 'blue',
		//shading: THREE.FlatShading,
	});

var materialblack = new THREE.MeshLambertMaterial(
{
		color: 'black',
		ambient: 'black',
		//shading: THREE.FlatShading,
});

var scale = 0.02;

var geometry = new THREE.BoxGeometry( 1.2, 0.03*0.1, 0.03*0.1 );
var cube = new THREE.Mesh( geometry, materialblack ); 
group.add(cube);
//scatter.objects.push(cube);

var geometry = new THREE.BoxGeometry( 0.03*0.1, 1.2, 0.03*0.1 );
var cube = new THREE.Mesh( geometry, materialblack ); 
group.add(cube); 
//scatter.objects.push(cube);

//cube.position.set(0, 0, 1.0);
//cube.updateMatrix();


for(var j=-5; j<=5; j++)
for(var i=-5; i<=5; i++) 
{
	var geometry = new THREE.SphereGeometry( 0.03, 8, 8 );
	var cube = new THREE.Mesh( geometry, material );

	var x0 = (scatter.a1.x*i + scatter.a2.x*j)*scale;
	var y0 = (scatter.a1.y*i + scatter.a2.y*j)*scale;
	var z0 = (scatter.a1.z*i + scatter.a2.z*j)*scale;

	var x1 = Math.sin(scatter.omega/180.*3.141592) * x0 + Math.cos(scatter.omega/180.*3.141592) * y0;
	var y1 = Math.cos(scatter.omega/180.*3.141592) * x0 - Math.sin(scatter.omega/180.*3.141592) * y0;
	var z1 = z0;
	
	var x2 = x1;
	var y2 = Math.sin(scatter.chi/180.*3.141592) * y1 + Math.cos(scatter.chi/180.*3.141592) * z1;
	var z2 = Math.cos(scatter.chi/180.*3.141592) * y1 - Math.sin(scatter.chi/180.*3.141592) * z1;

	var x3 = Math.cos(-scatter.eta/180.*3.141592) * x2 + Math.sin(-scatter.eta/180.*3.141592) * z2;
	var y3 = y2;
	var z3 = -Math.sin(-scatter.eta/180.*3.141592) * x2 +	Math.cos(-scatter.eta/180.*3.141592) * z2;
	cube.position.set(x3, y3, z3);
	group.add(cube);
	//scatter.objects.push(cube);

	/*
	if ((i^j)&1) {
		var arrow = CreateArrow(
			new THREE.Vector3(0,0,1), 
			new THREE.Vector3(i*scale, j*scale, -0.05), 0.1, 4, "#000080")
		scene.add(arrow);
	} else {
		var arrow = CreateArrow(
			new THREE.Vector3(0,0,-1), 
			new THREE.Vector3(i*scale, j*scale, 0.05), 0.1, 4, "#008080")
		scene.add(arrow);
	}
	*/
}
	this.objects.push(group);
	scene.add(group);

}

ScatterPlot.prototype.DrawIncomingLight = function()
{
	var dir = new THREE.Vector3( this.kx, this.ky, this.kz );
	var length = this.arrowlength*1.2;
	var origin = new THREE.Vector3( -length*this.kx, -length*this.ky, -length*this.kz+0. );
	var hex = 0xff0000;
	
	var arrow = CreateArrow( dir, origin, length, 1, hex);
	//arrow.cone.material = this.materialred;
	//arrow.line.material = this.materialred;

	this.objects.push(arrow);
	scene.add(arrow);
	//console.log(kx + " " + ky + " " + kz);
}


ScatterPlot.prototype.Update = function()
{
	this.DeleteArrows();
	this.CreateReciprocalLattice();
	this.RenderSample();
	
	// -----
	if (this.fixMiller)
	{
		this.CreateScatterSphere(this.diffh, this.diffk, this.diffl);
		var ret = this.SetThetaPhiHKL(this.diffh, this.diffk, this.diffl);
		if (!ret) return;
	}
	
	// -----
	this.CalcLightWavevector();
	
	if (this.showaxislabels) this.CreateAxisLabels();
	if (this.showpolarization) this.CreatePolarization();
	if (this.showsphere) this.CreateSphere();

	this.DrawIncomingLight();
	
	// ---------------
	var group = new THREE.Object3D();

	for(var i=0; i<this.G.length; i++)
	{
		var koutx = this.k0*this.kx + this.G[i].x;
		var kouty = this.k0*this.ky + this.G[i].y;
		var det = this.k0*this.k0 - koutx*koutx - kouty*kouty;
		if (det < 0) continue;
		var koutz = Math.sqrt(det);
		var dir = new THREE.Vector3(koutx, kouty, koutz);
		dir.normalize();
		var origin = new THREE.Vector3( 0., 0., 0. );
		length = 1;
		var hex = 0xff8000;

		if (!this.showsphere)
		{
			var arrow = CreateArrow(dir, origin, this.arrowlength, 1, hex);
			group.add(arrow);
		} else
		{
			dir.multiplyScalar(this.arrowlength);
			var geometry = new THREE.SphereGeometry( 0.01, 20, 20 ); 
			var sphere = new THREE.Mesh( geometry, materialblack ); 
			sphere.position.set(dir.x, dir.y, dir.z);
			group.add(sphere);
		}
		
		if (this.showrllabels) {
			var sprite = makeTextSprite( "       "+this.Ni[i]+" " +this.Nj[i], { fontsize: 80 } );
			//dir.multiplyScalar(this.arrowlength);
			sprite.position.set(dir.x*this.arrowlength, dir.y*this.arrowlength, dir.z*this.arrowlength);
			group.add( sprite );
		}

		if (!this.showtransmitted) continue;
		dir = new THREE.Vector3(koutx, kouty, -koutz);
		dir.normalize();
		origin = new THREE.Vector3(0., 0., -this.arrowthickness/2);
		length = 1;
		hex = 0xff8000;
		arrow = CreateArrow(dir, origin, this.arrowlength, 1, hex);
		group.add(arrow);
	}
	
	this.objects.push(group);
	scene.add(group);

}
